@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | PHP Version 5                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2013 The PHP Group                                |
+   | Copyright (c) 1997-2014 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -24,21 +24,26 @@
 static void _conn_close_cb(struct evhttp_connection *conn, void *arg)/* {{{ */
 {
 	php_event_http_conn_t *evcon = (php_event_http_conn_t *) arg;
+	zend_fcall_info       *pfci;
+	zend_fcall_info_cache *pfcc;
+	zval  *arg_data;
+	zval  *arg_conn;
+	zval **args[2];
+	zval  *retval_ptr = NULL;
+	PHP_EVENT_TSRM_DECL
+
 	PHP_EVENT_ASSERT(evcon && conn);
 
-	zend_fcall_info       *pfci = evcon->fci_closecb;
-	zend_fcall_info_cache *pfcc = evcon->fcc_closecb;
+	pfci = evcon->fci_closecb;
+	pfcc = evcon->fcc_closecb;
 	PHP_EVENT_ASSERT(pfci && pfcc);
 
-	TSRMLS_FETCH_FROM_CTX(evcon->thread_ctx);
+	PHP_EVENT_TSRMLS_FETCH_FROM_CTX(evcon->thread_ctx);
 
 	/* Call userspace function according to
 	 * proto void callback(EventHttpConnection conn, mixed data); */
 
-	zval  *arg_data = evcon->data_closecb;
-	zval  *arg_conn;
-	zval **args[2];
-	zval  *retval_ptr;
+	arg_data = evcon->data_closecb;
 
 	arg_conn = evcon->self;
 	if (conn == NULL || !arg_conn) {
@@ -96,6 +101,8 @@ PHP_METHOD(EventHttpConnection, __construct)
 		return;
 	}
 
+	PHP_EVENT_REQUIRE_BASE_BY_REF(zbase);
+
 	PHP_EVENT_FETCH_BASE(b, zbase);
 
 	if (zdns_base) {
